/*!
 * @file    uwb_SmartField.h
 * @author  Kryštof Vaněk <krystof.vanek@gmail.com>
 * @version 3.0
 * @date    2021-05-06
 * @brief   SmartField library user interface
 *
 * @attention <h2><center>&copy; Copyright (c) 2021 University of West Bohemia in Pilsen All rights reserved.</center></h2>
 * @attention <h2><center>Use of this source code without author permission and license agreement is prohibited. For license agreement contact the author.</center></h2>
 * @attention <h2><center>Tento zdrojový kód nelze využívat bez souhlasu autora a dohodě o licenčních podmínkách. Pro případné licencování kontaktujte autora.</center></h2>
 *
 * Developed by:<br>
 * University of West Bohemia<br>
 * Department of Technologies and Measurement<br>
 * www.zcu.cz
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 * OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 */

#ifndef _UWB_SMARTFIELD_H_
#define _UWB_SMARTFIELD_H_

#ifdef __cplusplus
extern "C" {
#endif

#define SMARTFIELD_ADC_BUFFER_LENGTH		4096
#define SMARTFIELD_FILT_BUFFER_LENGTH		2048
#define SMARTFIELD_MEAN_COUNT				5
#define SMARTFIELD_ADCMAX					2.6
#define SMARTFIELD_ADCMIN					0.6
#define SMARTFIELD_Q						((SMARTFIELD_ADCMAX-SMARTFIELD_ADCMIN)/256)
#define SMARTFIELD_CSNS						6e-12
#define SMARTFIELD_RSNS						50
//#define SMARTFIELD_ZSNS						49.991119722379
//#define SMARTFIELD_PHICORR					-9.012580734120409

#define SMARTFIELD_DEFAULT_MEASCOUNT		10
#define SMARTFIELD_DEFAULT_MEASPERIOD		10
#define SMARTFIELD_DEFAULT_NTCBETA			3940

#include "stdint.h"

typedef enum _SmartField_Status
	{
		SMARTFIELD_OK = 0,
		SMARTFIELD_ADC_SATURATION,
		SMARTFIELD_ADC_MALFUNCTION,
		SMARTFIELD_SYNCHRONIZATION_ERR,
		SMARTFIELD_ERR
	} SmartField_Status_t;

typedef struct _SmartField_Settings
	{
		uint8_t measPeriod;
		uint8_t measCount;
		uint16_t NTCbeta;
		float zCalibMultiplier;
		float zCalibOffset;
		float phiCalibMultiplier;
		float phiCalibOffset;
		float Zsns;
		float normalisedFrequency;
	} SmartField_Settings_t;

typedef struct _SmartField_Sample
	{
		uint8_t voltage;
		uint8_t current;
	} SmartField_Sample_t;

typedef struct _SmartField_Result
	{
		float VRMS;
		float IRMS;
		float angle;
		float _Complex ZC;
		float ntcResistance;
		float temperature;
	} SmartField_Result_t;

#include "stdio.h"
#include "stdbool.h"
#include "string.h"
#include "uwb_SmartField_conf.h"
#include "uwb_SmartField_IO.h"

#if SMARTFIELD_USE_MODULAR_IDENTIFICATION != 0
	#include "uwb_SmartField_Driver_Ident.h"
#endif /* SMARTFIELD_USE_MODULAR_IDENTIFICATION*/

#if SMARTFIELD_USE_IMM_MODEL_A != 0
	#include "uwb_SmartField_CS2200.h"
#endif /* SMARTFIELD_USE_IMM_MODEL_A*/

#define SMARTFIELD_NUMBER_OF_MODULES	4

/** @defgroup SmartField_BinHeaders SmartField Binary File headers
 * @brief SmartField Binary File header versions
 * @{
 */

/**
 * @brief Binary file header version 01
 */
typedef struct _SmartField_BinaryDataHeader_v01
	{
		uint8_t version;		/*!< Header version */
		uint8_t measCnt;		/*!< Number of measurements in file */
		uint16_t vectorLen;		/*!< Length of measurement vector (Single sample accounts for both voltage and current) */
	}__attribute__((packed)) SmartField_BinaryDataHeader_v01_t;

/**
 * @brief Binary file header version 02
 */
typedef struct _SmartField_BinaryDataHeader_v02
	{
		uint8_t version;		/*!< Header version */
		uint8_t measCnt;		/*!< Number of measurements in file */
		uint16_t vectorLen;		/*!< Length of measurement vector (Single sample accounts for both voltage and current) */
		uint8_t UID[12];		/*!< Microcontroller UID - used for data origin identification */
	}__attribute__((packed)) SmartField_BinaryDataHeader_v02_t;

/**
 * @brief Binary file header version 03
 */
typedef struct _SmartField_BinaryDataHeader_v03
	{
		uint8_t version;			/*!< Header version */
		uint8_t measCnt;			/*!< Number of measurements in file */
		uint16_t vectorLen;			/*!< Length of measurement vector (Single sample accounts for both voltage and current) */
		uint8_t UID[12];			/*!< Microcontroller UID - used for data origin identification */
		uint16_t NTCbeta;			/*!< Thermistor Beta parameter */
		float Zsns;					/*!< Magnitude of impedance in Current/Voltage converter feedback*/
		float normalisedFrequency;	/*!< Normalised frequency used for sampling (used by DFT) */
	}__attribute__((packed)) SmartField_BinaryDataHeader_v03_t;

/**
 * @brief Binary file header version 04
 */
typedef struct _SmartField_BinaryDataHeader_v04
	{
		uint8_t version;			/*!< Header version */
		uint8_t measCnt;			/*!< Number of measurements in file */
		uint16_t vectorLen;			/*!< Length of measurement vector (Single sample accounts for both voltage and current) */
		uint8_t UID[12];			/*!< Microcontroller UID - used for data origin identification */
		uint16_t NTCbeta;			/*!< Thermistor Beta parameter */
		float Zsns;					/*!< Magnitude of impedance in Current/Voltage converter feedback*/
		float normalisedFrequency;	/*!< Normalised frequency used for sampling (used by DFT) */
		uint8_t numberOfSlots;		/*!< Number of modular slots (excludes controller); 0 in case of non-modular implementation */
		uint8_t numberOfChannels;	/*!< Number of measurement channels (usable measurement modules) */
	}__attribute__((packed)) SmartField_BinaryDataHeader_v04_t;

/**
 * @}
 */

typedef struct _SmartField_Handle
	{
	#if SMARTFIELD_USE_MODULAR_IDENTIFICATION != 0
		SmartField_Module_ID_t platformSetup[SMARTFIELD_NUMBER_OF_MODULES];
	#endif /* SMARTFIELD_USE_MODULAR_IDENTIFICATION*/
		SmartField_Settings_t Settings;
		void (*dataDumpCallback)(void* pData, uint32_t dataLength);
	} SmartField_Handle_t;

//extern SmartField_Handle_t hSmartField;

extern void SmartField_Init(SmartField_Handle_t* hSmartField);
extern void SmartField_Measure(SmartField_Handle_t* hSmartField);
extern SmartField_Result_t* SmartField_getAverage();

#ifdef __cplusplus
}
#endif

#endif /* _UWB_SMARTFIELD_H_ */
