#include "smartfield_fileloader.h"
#include <QFile>

typedef struct __SmartField_BinaryDataHeader_v01 {
    uint8_t version;
    uint8_t measCnt;
    uint16_t vectorLen;
} SmartField_BinaryDataHeader_v01_t;

typedef struct __SmartField_BinaryDataHeader_v02 {
    uint8_t version;
    uint8_t measCnt;
    uint16_t vectorLen;
    uint8_t UID[12];
} SmartField_BinaryDataHeader_v02_t;

typedef struct {
    uint8_t version;
    uint8_t measCnt;
    uint16_t vectorLen;
    uint8_t UID[12];
    uint16_t NTCbeta;
    float Zsns;
    float normalisedFrequency;
} __attribute__((packed)) SmartField_BinaryDataHeader_v03_t;

typedef union __SmartField_BinaryDataHeader {
    SmartField_BinaryDataHeader_v01_t header_v01;
    SmartField_BinaryDataHeader_v02_t header_v02;
    SmartField_BinaryDataHeader_v03_t header_v03;
} SmartField_BinaryDataHeader_t;

void SmartField_FileLoader::loadFile(QFile* inFile)
{
    if (!inFile->open(QIODevice::ReadOnly))
        return;

    QByteArray fileContent = inFile->readAll();
    inFile->close();

    processHeader(&fileContent);

    /* Development purpose only!!! */
    if (numberOfMeasurements == 0) {
        numberOfMeasurements = 1;
    }

    int voltageOffset = 0;
    int currentOffset = 1;
    for (int measurement = 0; measurement < numberOfMeasurements; measurement++) {
        QVector<uint8_t> tmpVoltage, tmpCurrent;
        int startOffset = 2 * numberofPoints * measurement;
        for (int i = startOffset; i < startOffset + numberofPoints * 2; i += 2) {
            tmpVoltage.append((uint8_t)fileContent[i + voltageOffset]);
            tmpCurrent.append((uint8_t)fileContent[i + currentOffset]);
        }
        voltage.append(tmpVoltage);
        current.append(tmpCurrent);
    }
}

float SmartField_FileLoader::getHeaderVersion()
{
    return headerVersion;
}
float SmartField_FileLoader::getNumberOfMeasurements()
{
    return numberOfMeasurements;
}
float SmartField_FileLoader::getNumberOfPoints()
{
    return numberofPoints;
}
QString SmartField_FileLoader::getUID()
{
    if (headerVersion == 1) {
        return QString("N/A");
    }
    return UID;
}
float SmartField_FileLoader::getNTCbeta()
{
    return NTCbeta;
}
float SmartField_FileLoader::getZsns()
{
    return Zsns;
}
float SmartField_FileLoader::getNormalisedFreqeuncy()
{
    return normalisedFrequency;
}

void SmartField_FileLoader::processHeader(QByteArray* fileContent)
{
    /* Copy file header */
    ulong headerSize;
    switch (fileContent->at(0)) {
    case 1:
        headerSize = sizeof(SmartField_BinaryDataHeader_v01_t);
        break;
    case 2:
        headerSize = sizeof(SmartField_BinaryDataHeader_v02_t);
        break;
    case 3:
        headerSize = sizeof(SmartField_BinaryDataHeader_v03_t);
        break;
    default:
        return;
    }
    SmartField_BinaryDataHeader_t header;
    for (ulong i = 0; i < headerSize; i++) {
        ((uint8_t*)&header)[i] = (uint8_t)fileContent->at(i);
    }
    headerVersion = header.header_v01.version;
    numberofPoints = header.header_v01.vectorLen;
    numberOfMeasurements = header.header_v01.measCnt;
    if (headerVersion > 1) {
        uint8_t* pUID = header.header_v02.UID;
        UID = QString("%1").arg(pUID[0], 2, 16, QLatin1Char('0'));
        for (int i = 1; i < 12; i++) {
            UID = UID + "-" + QString("%1").arg(pUID[i], 2, 16, QLatin1Char('0'));
        }
    }

    if (headerVersion > 2) {
        normalisedFrequency = header.header_v03.normalisedFrequency;
        Zsns = header.header_v03.Zsns;
    } else {
        normalisedFrequency = 0.00164065622;
        Zsns = 69.703537726290079;
    }

    fileContent->remove(0, headerSize);
}

void SmartField_FileLoader::reset()
{
    headerVersion = NAN;
    numberOfMeasurements = NAN;
    numberofPoints = NAN;
    UID = "";
    NTCbeta = NAN;
    Zsns = NAN;
    normalisedFrequency = NAN;
    voltage.clear();
    current.clear();
}
